package Opt;

##
##	This is the `Opt.pm' module for reading parameters from command line & file
##
##	Thu Mar 29 08:54:06 CEST 2001
##	
##	By Jiri Polcar (polcar@physics.muni.cz).
##

use Exporter;
use Getopt::Long;

use strict;

@main::ISA = qw(Exporter);
@main::EXPORT_OK = qw( getOptionsFromCmdLine  getOptionsFromFile );

my $true = 1;
my $false = 0;


# read options from command line; 
sub getOptionsFromCmdLine {
	my ( $opt ) = @_;

	my @to_get_options;
	my $key;
	my $realy_param;
	my %options;


	@to_get_options = ();
	foreach $key ( keys( %{$opt} ) ) {
		{
			if ( $opt->{$key}->{param} =~ /\@$/ ) {								# array
				my @local_key;
				@local_key = ();
				$options{$key} = \@local_key;
			} elsif ( $opt->{$key}->{param} =~ /\%$/ ) {					# hash
				my %local_key;
				%local_key = ();
				$options{$key} = \%local_key;
			} else {																							#	scalar
				my $local_key;
				$local_key = $opt->{$key}->{value};
				$options{$key} = $local_key;
			}
			push( @to_get_options, $key.$opt->{$key}->{alias}.$opt->{$key}->{param});
		}
	}
	Getopt::Long::Configure ("bundling");
	unless ( GetOptions( \%options, @to_get_options ) ) { return undef; }
	updateDefaultsOptions( \%options, $opt );	
	return $true;
}


#	read options ( `$opt' ) from config file `$file'; can read scalar, array and
#	hash value; works only with single lines!!!
sub getOptionsFromFile {
	my ( $file, $opt ) = @_;

	unless ( open( C_FILE, "<$file" ) ) {
#		print STDERR "\nWARNING: Cannot read `$file' file!\n";
		return $false;
	}

	my @param;
	my $line;
	my %options;
	my $array_value;
	my $del_array;
	my @key_value;
	
	$line = 0;
	$del_array = $true;
	while ( <C_FILE> ) {
		$line++;
		chomp;
		if ( ( /^\s*([^\#]*)[\#]*\s*/ ) && $1 ) {	#	diskart empty and `#' files
			$_ = $1;
			s/[\s=]+/ /g;														# string string string...
			@param = split(/\s/, $_ );
			if ( exists( $opt->{$param[0]} ) ) {
				$_ = ref( $opt->{$param[0]}->{value} );
				if ( /ARRAY/ )	{
					unless ( defined( $array_value = 
													updateScalar( $line, $file, $opt, @param) ) ) {
						return undef;
					} else { push( @{$options{$param[0]}}, $array_value ); }	
				} elsif ( /HASH/ )	{
					( @key_value ) = updateHash( $line, $file, $opt, @param);
					unless ( defined( $key_value[0]) && defined( $key_value[1]) ) {
						return undef;
					} else { 
 						${$options{$param[0]}}{$key_value[0]} =  $key_value[1];
					}
				} else	{ 
					unless ( defined( $options{$param[0]} = 
													updateScalar( $line, $file, $opt, @param) ) ) {
						return undef;
					}
				}
			} else {
				print STDERR "WARNING: Error in config file ($file) at $line line.\n";
				next;
			}
		}
	}
	close( C_FILE );
	updateDefaultsOptions( \%options, $opt );	
	return $true;
}


#	needy to `getOptionsFromFile'; read scalar value; return UNDEF if unsuccess
sub updateScalar {
	my ( $line, $file, $opt, @param ) = @_;

	$_ = $param[1];
	if ( $opt->{$param[0]}->{param} =~ /\=/ ) {
		unless ( defined( $_ ) ) {
			print STDERR "Expect value at line $line in config ($file).\n";
			return undef;
		}
		if ( $opt->{$param[0]}->{param} =~ /i/i ) {
			unless ( /^[-+]?\d+$/) {
				print STDERR "Expect integer value at line $line in config ($file).\n";
				return undef;
			} else { return $_; }
		} elsif ( $opt->{$param[0]}->{param} =~ /f/i ) {
			unless ( /^[-+]?\d*\.?\d*$/) {
				print STDERR "Expect float value at line $line in config ($file).\n";
				return undef;
			} else { return $_; }
		} else { return $_;}
	} else { return $true; }
}

#	needy to `getOptionsFromFile'; read hash value; return (UNDEF, UNDEF) if
#	unsuccess
sub updateHash {
	my ( $line, $file, $opt, @param ) = @_;

	my $key; 
	my $value;

	$key = $param[1];
	$value = $param[2];
	
	unless ( defined( $key) && defined( $value ) ) {
		print STDERR "Hash parametr expects key and value at line ".
								 "$line in config ($file).\n";
		return (undef, undef);
	} else {
		return ( $key, $value );
	}
}


sub updateDefaultsOptions {
	my ( $options, $opt ) = @_;			#	options	:	siple hash with new values
																	#	opt			:	large definition...
	my $key;

	my $tmp;
	my $something;

	foreach $key (keys %{$opt} ) {
		if ( exists( $options->{$key} ) ) {
			$something = $false;
			$_ = ref($options->{$key});

			if ( /HASH/ ) {	
				foreach $tmp ( keys %{$options->{$key}} ) { $something = $true; }
				if ( $something ) {	$opt->{$key}->{value} = $options->{$key}; }
			}	elsif ( /ARRAY/ ) {	
				foreach $tmp ( @{$options->{$key}} ) { $something = $true; }
				if ( $something ) { $opt->{$key}->{value} = $options->{$key}; }
			} else {
				$opt->{$key}->{value} = $options->{$key}; 
			}
		} else { next; } 
	}
	return $true;
}


$true;
